﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Data.SqlClient;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Wellcode.Basics;
using Wellcode.Frame;

namespace AdventureWorks
{
    public partial class BrowserForm : Form
    {
        public BrowserForm()
        {
            InitializeComponent();
        }

        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            var selected = -1;
            var members = TypeUtil.GetMembers(typeof(AdvEntityManager));
            foreach (var mem in members)
            {
                if (false == mem.Key.EndsWith("Factory"))
                    continue;

                if (mem.Key == "PersonFactory")
                    selected = this.treeRootComboBox.Items.Count;

                this.treeRootComboBox.Items.Add(mem.Key.Replace("Factory", ""));
            }
            if (selected > -1)
            {
                this.treeRootComboBox.SelectedIndex = selected;
            }
        }

        private void cmdTestConnection_Click(object sender, EventArgs e)
        {
            this.Cursor = Cursors.WaitCursor;
            try
            {
                if (string.IsNullOrEmpty(cmbDatabase.Text) || string.IsNullOrEmpty(cmbServer.Text))
                {
                    MessageBox.Show("Enter database and server name and try again");
                    return;
                }

                // add columns to grid
                tree.Nodes.Clear();

                // set default connection string
                var connString = GetConnection().ConnectionString;
                Wellcode.Basics.Config.Instanz.DataConnection = new DataConnection()
                {
                    Number = "adv",
                    ConnectionString = connString,
                };

                // revert all changes
                AdvEntityManager.Manager.Reset();


                var entities = TypeUtil.Invoke(AdvEntityManager.Manager, "Get" + treeRootComboBox.Text + "List");
                foreach (var item in (IEnumerable)entities)
                {
                    this.AddWebEntity(tree.Nodes, treeRootComboBox.Text, (WebEntity)item);
                    if (tree.Nodes.Count >= this.MaxEntries)
                        break;
                }

                // changes can be saved
                this.saveButton.Enabled = false;
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error occured\r\n-----\r\n" + ex.ToString());
            }
            finally
            {
                this.Cursor = Cursors.Default;
            }
        }

        private void saveButton_Click(object sender, EventArgs e)
        {
            try
            {
                AdvEntityManager.Manager.SaveAll();
                saveButton.Enabled = false;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            }
        }

        private SqlConnectionStringBuilder GetConnection()
        {
            var sql = new SqlConnectionStringBuilder();

            sql.DataSource = cmbServer.Text;
            sql.InitialCatalog = cmbDatabase.Text;

            if (radWindowsLogin.Checked)
            {
                sql.IntegratedSecurity = true;
            }
            else
            {
                sql.UserID = txtUser.Text;
                sql.Password = txtPassword.Text;
            }

            return sql;
        }

        private void tree_BeforeExpand(object sender, TreeViewCancelEventArgs e)
        {
            if (e.Node.Nodes.Count != 1 || e.Node.Nodes[0].Tag != null)
                return;

            e.Node.Nodes.Clear();

            var entity = (WebEntity)e.Node.Tag;
            var members = TypeUtil.GetMembers(entity.GetType(), false);
            foreach (var mem in members)
            {
                object value = null;
                try
                {
                    value = TypeUtil.GetValue(entity, mem.Key);
                }
                catch
                {
                    // ignore
                }
                if (value == null)
                    continue;

                // foreign keys (sub entities) 
                if (value is IWebEntityList)
                {
                    var subNode = e.Node.Nodes.Add(mem.Key);
                    subNode.Tag = value;
                    foreach (WebEntity item in (IEnumerable)value)
                    {
                        this.AddWebEntity(subNode.Nodes, mem.Key.Replace("List", ""), item);
                        if (subNode.Nodes.Count >= this.MaxEntries)
                            break;
                    }
                }

                // constraints (parent entities)
                if (value is WebEntity)
                {
                    var item = (WebEntity)value;
                    this.AddWebEntity(e.Node.Nodes, mem.Key, item);
                }
            }
        }

        int MaxEntries
        {
            get
            {
                try
                {
                    return Convert.ToInt32(this.maxEntriesTextbox.Text);
                }
                catch
                {
                    return 1000;
                }
            }
        }

        void AddWebEntity(TreeNodeCollection nodes, string propName, WebEntity item)
        {
            var subNode = nodes.Add(propName + " [" + item.Values[0].ToString() + " " + item.Values[1].ToString() + " " + item.Values[2].ToString() + "]");
            subNode.Tag = item;
            subNode.Nodes.Add("DUMMY", "DUMMY");
        }

        private void tree_AfterSelect(object sender, TreeViewEventArgs e)
        {
            WebEntity entity = null;

            lv.Clear();
            lv.Columns.Clear();
            if (e.Node.Tag is IWebEntityList)
            {
                var list = (IWebEntityList)e.Node.Tag;
                if (list.Count == 0)
                    return;

                split2.Panel2Collapsed = false;

                foreach (var item in list.WebEntities)
                {
                    foreach (var pair in item.Values)
                    {
                        if (!lv.Columns.ContainsKey(pair.Key))
                        {
                            lv.Columns.Add(pair.Key, pair.Key);
                        }
                    }

                    var row = lv.Items.Add(Convert.ToString(item.Values[0]));
                    foreach (var pair in item.Values)
                    {
                        row.SubItems.Add(Convert.ToString(pair.Value));
                    }
                }

                lv.AutoResizeColumns(ColumnHeaderAutoResizeStyle.ColumnContent);
                lv.AutoResizeColumns(ColumnHeaderAutoResizeStyle.HeaderSize );

                if (e.Node.Parent.Tag is WebEntity)
                    entity = (WebEntity)e.Node.Parent.Tag;
            }
            else
            {
                split2.Panel2Collapsed = true;
            }

            propPanel.Controls.Clear();

            if (entity == null)
                entity = (WebEntity)e.Node.Tag;

            if (entity == null)
                return;

            propPanel.Tag = entity;
            this.entityNameLabel.Text = entity.GetType().Name;

            var y = 5;
            var x = 5;
            foreach (var pair in entity.Values)
            {
                var lbl = new Label()
                {
                    Top = y,
                    Left = x,
                    Text = pair.Key
                };
                var txt = new TextBox()
                {
                    Top = y,
                    Left = x + 200,
                    Text = pair.ToString(),
                    Width = propPanel.Width - x - 210,
                    Tag = pair,
                };
                txt.TextChanged += txt_TextChanged;
                propPanel.Controls.Add(lbl);
                propPanel.Controls.Add(txt);

                y += txt.Height + 5;
            }
        }

        void txt_TextChanged(object sender, EventArgs e)
        {
            var txt = ((TextBox)sender);
            var pair = (Wellcode.Frame.ValuePair)txt.Tag;
            ((WebEntity)propPanel.Tag)[pair.Key] = txt.Text;
            this.saveButton.Enabled = true;
        }

        private void BrowserForm_Load(object sender, EventArgs e)
        {

        }
    }
}
