﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Data.SqlClient;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Wellcode.Basics;
using Wellcode.Frame;

namespace AdventureWorks
{
    public partial class PersonForm : Form
    {
        public PersonForm()
        {
            InitializeComponent();
        }

        private void cmdTestConnection_Click(object sender, EventArgs e)
        {
            this.Cursor = Cursors.WaitCursor;
            try
            {
                if (string.IsNullOrEmpty(cmbDatabase.Text) || string.IsNullOrEmpty(cmbServer.Text))
                {
                    MessageBox.Show("Enter database and server name and try again");
                    return;
                }

                // set default connection string
                var connString = GetConnection().ConnectionString;
                Wellcode.Basics.Config.Instanz.DataConnection.ConnectionString = connString;

                // revert all changes
                AdvEntityManager.Manager.Reset();

                // build filter where clause, if any
                var where = new SqlWhereClause();
                if(!string.IsNullOrEmpty( this.filterTextBox.Text))
                {
                    var pattern = "%" + this.filterTextBox.Text + "%";
                    where.Clause = "WHERE FirstName LIKE @FirstName OR LastName = @LastName";
                    where.Add("@FirstName", pattern);
                    where.Add("@LastName", pattern);
                }
                where.Clause += " ORDER BY FirstName, LastName";

                // get all persons matching above where clause
                var persons = AdvEntityManager.Manager.GetPersonList(where);

                // add person columns to grid
                if (grid.Columns.Count == 0 && persons.Count > 0)
                    foreach(var pair in persons[0].Values)
                    {
                        grid.Columns.Add(pair.Key, pair.Key);
                    }

                // clear previous contents
                if (grid.Columns.Count > 0)
                    grid.Rows.Clear();

                // display all found persons
                foreach(var person in persons)
                {
                    var r = grid.Rows.Add(person.Values);
                    grid.Rows[r].Tag = person;
                }

                // changes can be saved
                //this.saveButton.Enabled = true;
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error occured\r\n-----\r\n" + ex.ToString());
            }
            finally
            {
                this.Cursor = Cursors.Default;
            }
        }

        // build connection string
        private SqlConnectionStringBuilder GetConnection()
        {
            var sql = new SqlConnectionStringBuilder();

            sql.DataSource = cmbServer.Text;
            sql.InitialCatalog = cmbDatabase.Text;

            if (radWindowsLogin.Checked)
            {
                sql.IntegratedSecurity = true;
            }
            else
            {
                sql.UserID = txtUser.Text;
                sql.Password = txtPassword.Text;
            }

            return sql;
        }

        private void grid_DataError(object sender, DataGridViewDataErrorEventArgs e)
        {
            e.ThrowException = false;
        }

        /// <summary>
        /// apply changes to the person entity object
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void grid_CellEndEdit(object sender, DataGridViewCellEventArgs e)
        {
            var row = grid.Rows[e.RowIndex];
            if (row.Tag == null)
            {
                row.Tag = AdvEntityManager.Manager.PersonFactory.CreateEntity();
            }
            var person = (Person)row.Tag;
            person[grid.Columns[e.ColumnIndex].Name] = row.Cells[e.ColumnIndex].Value;

            saveButton.Enabled = AdvEntityManager.Manager.HaveChanges();
        }

        /// <summary>
        /// prevent changing primary index
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void grid_CellBeginEdit(object sender, DataGridViewCellCancelEventArgs e)
        {
            if (e.ColumnIndex == 0 && false == grid.Rows[e.RowIndex].IsNewRow)
            {
                MessageBox.Show("Primary Field can not be changed");
                e.Cancel = true;
            }
        }

        /// <summary>
        /// save all changes, if any
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void saveButton_Click(object sender, EventArgs e)
        {
            try
            {
                AdvEntityManager.Manager.SaveAll();
                saveButton.Enabled = false;
            }
            catch(Exception ex)
            {
                MessageBox.Show(ex.ToString());
            }
        }
    }
}
