﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.ServiceModel.Web;
using System.Text;
using System.Threading;
using Wellcode.Basics;
using Wellcode.Frame;

namespace AdventureWorks
{
    // HINWEIS: Mit dem Befehl "Umbenennen" im Menü "Umgestalten" können Sie den Klassennamen "Service1" sowohl im Code als auch in der SVC- und der Konfigurationsdatei ändern.
    // HINWEIS: Wählen Sie zum Starten des WCF-Testclients zum Testen dieses Diensts Service1.svc oder Service1.svc.cs im Projektmappen-Explorer aus, und starten Sie das Debuggen.
    public partial class AdventureWorksService 
        : BasicSynchronizer
        , IAdventureWorksService
    {
        public string GetData(int value)
        {
            return string.Format("You entered: {0}", value);
        }

        public CompositeType GetDataUsingDataContract(CompositeType composite)
        {
            if (composite == null)
            {
                throw new ArgumentNullException("composite");
            }
            if (composite.BoolValue)
            {
                composite.StringValue += "Suffix";
            }
            return composite;
        }

        #region Execution
        private class ServiceExecuter
            : IDisposable
        {
            static int interlockNumber = 0;
            public ServiceExecuter()
            {
                while (0 != Interlocked.Exchange(ref interlockNumber, 1))
                    Thread.Sleep(1);
            }

            /// <summary>
            /// Skript-Definition, um unnamed Funktionen (Lambda Ausdrücke) auszuführen
            /// </summary>
            public delegate void Script();

            /// <summary>
            /// Skript wird synchronisiert ausgeführt
            /// </summary>
            /// <param name="cmd"></param>
            public static void Execute(Script cmd)
            {
                try
                {
                    EntityManager.Manager.Reset();
                    cmd();
                }
                finally
                {
                    EntityManager.Manager.Reset();
                }
            }

            /// <summary>
            /// Skript wird asynchron ausgeführt
            /// </summary>
            /// <param name="cmd"></param>
            public static void ExecuteAsync(Script cmd)
            {
                try
                {
                    EntityManager.Manager.Reset();
                    cmd();
                }
                finally
                {
                    EntityManager.Manager.Reset();
                }
            }


            public void Dispose()
            {
                // release the lock
                Interlocked.Exchange(ref interlockNumber, 0);
            }
        }


        /// <summary>
        /// lambda script, which is executed
        /// </summary>
        public delegate void ServiceScript(AdvEntityManager manager);

        /// <summary>
        /// Wrapper for all service calls
        /// </summary>
        /// <param name="cmd"></param>
        public virtual void Execute(ServiceScript cmd)
        {
            // marker, we are in web service
            WebServiceConfig.IsWebServiceActive = true;

            var manager = (AdvEntityManager)OperationContext.Current.RequestContext.RequestMessage.Properties["EntityManager"];
            this.Enter();
            try
            {
                cmd(manager);
            }
            finally
            {
                this.Leave();
                //manager.Dispose();
            }
        }
        #endregion
    }
}
