﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using System.Web;
using System.ServiceModel.Web;

using Wellcode.Basics;
using Wellcode.Frame;

namespace AdventureWorks
{
    public partial interface IAdventureWorksService
    {
		/// <summary>
        /// getting existing entity 'BusinessEntity' using it's primary key(s)
		/// Tabelle BusinessEntity
        /// </summary>
        /// <returns>a new object of entity 'BusinessEntity', <see href="ResultInfo.IsValid"/></returns>
        [OperationContract]
		[WebInvoke(UriTemplate = "/GetBusinessEntity", Method = "POST", RequestFormat = WebMessageFormat.Json, ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
		[ServiceKnownType("GetKnownTypes", typeof(KnownTypeProvider))]
        BusinessEntity GetBusinessEntity(int businessEntityId);

        /// <summary>
        /// get list of entities 'BusinessEntityList' using a Where clause
		/// Database Table BusinessEntity
        /// </summary>
        /// <returns>Entity 'BusinessEntityList' with matched result</returns>
        [OperationContract]
		[WebInvoke(UriTemplate = "/GetBusinessEntityList", RequestFormat = WebMessageFormat.Json, ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
		[ServiceKnownType("GetKnownTypes", typeof(KnownTypeProvider))]
        BusinessEntityList GetBusinessEntityList(SqlWhereClause where);

        /// <summary>
        /// Save or create an entity 'BusinessEntity', accourding to entity content and state.
		/// Database Table BusinessEntity
        /// </summary>
        /// <returns>Entity 'BusinessEntity', the saved/created entity, proof <see href="ResultInfo.IsValid"/></returns>
        [OperationContract]
		[WebInvoke(UriTemplate = "/SaveBusinessEntity", RequestFormat = WebMessageFormat.Json, ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
		[ServiceKnownType("GetKnownTypes", typeof(KnownTypeProvider))]
        BusinessEntity SaveBusinessEntity(BusinessEntity item);

        /// <summary>
        /// Remove existing entity 'BusinessEntity'
		/// Database Table BusinessEntity
        /// </summary>
        /// <returns>true, when removed, false otherwise</returns>
        [OperationContract]
		[WebInvoke(UriTemplate = "/RemoveBusinessEntity", RequestFormat = WebMessageFormat.Json, ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
		[ServiceKnownType("GetKnownTypes", typeof(KnownTypeProvider))]
        bool RemoveBusinessEntity(BusinessEntity item);
    }

    public partial class AdventureWorksService
    {
        /// <summary>
        /// <see href="IAdventureWorksService.GetBusinessEntity" />
		/// Database Table BusinessEntity
        /// </summary>
        virtual public BusinessEntity GetBusinessEntity(int businessEntityId)
        {
		    BusinessEntity result = null;
			this.Execute((manager) =>
			{
				result = manager.GetBusinessEntity(businessEntityId);
			});
			return result;
        }

        /// <summary>
        /// <see href="IAdventureWorksService.GetBusinessEntityList" />
		/// Database Table BusinessEntity
        /// </summary>
        virtual public BusinessEntityList GetBusinessEntityList(SqlWhereClause where)
        {
			BusinessEntityList result = null;
			this.Execute((manager) =>
			{
				result = manager.GetBusinessEntityList(where);
			});
			return result;
        }

        /// <summary>
        /// <see href="IAdventureWorksService.SaveBusinessEntity" />
		/// Database Table BusinessEntity
        /// </summary>
        virtual public BusinessEntity SaveBusinessEntity(BusinessEntity item)
        {
			BusinessEntity result = null;
			this.Execute((manager) =>
			{
				result = manager.SaveBusinessEntity(item);
			});
			return result;
        }
		
        /// <summary>
        /// <see href="IAdventureWorksService.RemoveBusinessEntity" />
		/// Database Table BusinessEntity
        /// </summary>
        virtual public bool RemoveBusinessEntity(BusinessEntity item)
        {
			bool result = false;
			this.Execute((manager) =>
			{
				result = manager.RemoveBusinessEntity(item);
			});
			return result;
        }
    }
}