﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using System.Web;
using System.ServiceModel.Web;

using Wellcode.Basics;
using Wellcode.Frame;

namespace AdventureWorks
{
    public partial interface IAdventureWorksService
    {
		/// <summary>
        /// getting existing entity 'Customer' using it's primary key(s)
		/// Tabelle Customer
        /// </summary>
        /// <returns>a new object of entity 'Customer', <see href="ResultInfo.IsValid"/></returns>
        [OperationContract]
		[WebInvoke(UriTemplate = "/GetCustomer", Method = "POST", RequestFormat = WebMessageFormat.Json, ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
		[ServiceKnownType("GetKnownTypes", typeof(KnownTypeProvider))]
        Customer GetCustomer(int customerId);

        /// <summary>
        /// get list of entities 'CustomerList' using a Where clause
		/// Database Table Customer
        /// </summary>
        /// <returns>Entity 'CustomerList' with matched result</returns>
        [OperationContract]
		[WebInvoke(UriTemplate = "/GetCustomerList", RequestFormat = WebMessageFormat.Json, ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
		[ServiceKnownType("GetKnownTypes", typeof(KnownTypeProvider))]
        CustomerList GetCustomerList(SqlWhereClause where);

        /// <summary>
        /// Save or create an entity 'Customer', accourding to entity content and state.
		/// Database Table Customer
        /// </summary>
        /// <returns>Entity 'Customer', the saved/created entity, proof <see href="ResultInfo.IsValid"/></returns>
        [OperationContract]
		[WebInvoke(UriTemplate = "/SaveCustomer", RequestFormat = WebMessageFormat.Json, ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
		[ServiceKnownType("GetKnownTypes", typeof(KnownTypeProvider))]
        Customer SaveCustomer(Customer item);

        /// <summary>
        /// Remove existing entity 'Customer'
		/// Database Table Customer
        /// </summary>
        /// <returns>true, when removed, false otherwise</returns>
        [OperationContract]
		[WebInvoke(UriTemplate = "/RemoveCustomer", RequestFormat = WebMessageFormat.Json, ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
		[ServiceKnownType("GetKnownTypes", typeof(KnownTypeProvider))]
        bool RemoveCustomer(Customer item);
    }

    public partial class AdventureWorksService
    {
        /// <summary>
        /// <see href="IAdventureWorksService.GetCustomer" />
		/// Database Table Customer
        /// </summary>
        virtual public Customer GetCustomer(int customerId)
        {
		    Customer result = null;
			this.Execute((manager) =>
			{
				result = manager.GetCustomer(customerId);
			});
			return result;
        }

        /// <summary>
        /// <see href="IAdventureWorksService.GetCustomerList" />
		/// Database Table Customer
        /// </summary>
        virtual public CustomerList GetCustomerList(SqlWhereClause where)
        {
			CustomerList result = null;
			this.Execute((manager) =>
			{
				result = manager.GetCustomerList(where);
			});
			return result;
        }

        /// <summary>
        /// <see href="IAdventureWorksService.SaveCustomer" />
		/// Database Table Customer
        /// </summary>
        virtual public Customer SaveCustomer(Customer item)
        {
			Customer result = null;
			this.Execute((manager) =>
			{
				result = manager.SaveCustomer(item);
			});
			return result;
        }
		
        /// <summary>
        /// <see href="IAdventureWorksService.RemoveCustomer" />
		/// Database Table Customer
        /// </summary>
        virtual public bool RemoveCustomer(Customer item)
        {
			bool result = false;
			this.Execute((manager) =>
			{
				result = manager.RemoveCustomer(item);
			});
			return result;
        }
    }
}