﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using System.Web;
using System.ServiceModel.Web;

using Wellcode.Basics;
using Wellcode.Frame;

namespace AdventureWorks
{
    public partial interface IAdventureWorksService
    {
		/// <summary>
        /// getting existing entity 'Document' using it's primary key(s)
		/// Tabelle Document
        /// </summary>
        /// <returns>a new object of entity 'Document', <see href="ResultInfo.IsValid"/></returns>
        [OperationContract]
		[WebInvoke(UriTemplate = "/GetDocument", Method = "POST", RequestFormat = WebMessageFormat.Json, ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
		[ServiceKnownType("GetKnownTypes", typeof(KnownTypeProvider))]
        Document GetDocument(string documentNode);

        /// <summary>
        /// get list of entities 'DocumentList' using a Where clause
		/// Database Table Document
        /// </summary>
        /// <returns>Entity 'DocumentList' with matched result</returns>
        [OperationContract]
		[WebInvoke(UriTemplate = "/GetDocumentList", RequestFormat = WebMessageFormat.Json, ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
		[ServiceKnownType("GetKnownTypes", typeof(KnownTypeProvider))]
        DocumentList GetDocumentList(SqlWhereClause where);

        /// <summary>
        /// Save or create an entity 'Document', accourding to entity content and state.
		/// Database Table Document
        /// </summary>
        /// <returns>Entity 'Document', the saved/created entity, proof <see href="ResultInfo.IsValid"/></returns>
        [OperationContract]
		[WebInvoke(UriTemplate = "/SaveDocument", RequestFormat = WebMessageFormat.Json, ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
		[ServiceKnownType("GetKnownTypes", typeof(KnownTypeProvider))]
        Document SaveDocument(Document item);

        /// <summary>
        /// Remove existing entity 'Document'
		/// Database Table Document
        /// </summary>
        /// <returns>true, when removed, false otherwise</returns>
        [OperationContract]
		[WebInvoke(UriTemplate = "/RemoveDocument", RequestFormat = WebMessageFormat.Json, ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
		[ServiceKnownType("GetKnownTypes", typeof(KnownTypeProvider))]
        bool RemoveDocument(Document item);
    }

    public partial class AdventureWorksService
    {
        /// <summary>
        /// <see href="IAdventureWorksService.GetDocument" />
		/// Database Table Document
        /// </summary>
        virtual public Document GetDocument(string documentNode)
        {
		    Document result = null;
			this.Execute((manager) =>
			{
				result = manager.GetDocument(documentNode);
			});
			return result;
        }

        /// <summary>
        /// <see href="IAdventureWorksService.GetDocumentList" />
		/// Database Table Document
        /// </summary>
        virtual public DocumentList GetDocumentList(SqlWhereClause where)
        {
			DocumentList result = null;
			this.Execute((manager) =>
			{
				result = manager.GetDocumentList(where);
			});
			return result;
        }

        /// <summary>
        /// <see href="IAdventureWorksService.SaveDocument" />
		/// Database Table Document
        /// </summary>
        virtual public Document SaveDocument(Document item)
        {
			Document result = null;
			this.Execute((manager) =>
			{
				result = manager.SaveDocument(item);
			});
			return result;
        }
		
        /// <summary>
        /// <see href="IAdventureWorksService.RemoveDocument" />
		/// Database Table Document
        /// </summary>
        virtual public bool RemoveDocument(Document item)
        {
			bool result = false;
			this.Execute((manager) =>
			{
				result = manager.RemoveDocument(item);
			});
			return result;
        }
    }
}