﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using System.Web;
using System.ServiceModel.Web;

using Wellcode.Basics;
using Wellcode.Frame;

namespace AdventureWorks
{
    public partial interface IAdventureWorksService
    {
		/// <summary>
        /// getting existing entity 'Employee' using it's primary key(s)
		/// Tabelle Employee
        /// </summary>
        /// <returns>a new object of entity 'Employee', <see href="ResultInfo.IsValid"/></returns>
        [OperationContract]
		[WebInvoke(UriTemplate = "/GetEmployee", Method = "POST", RequestFormat = WebMessageFormat.Json, ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
		[ServiceKnownType("GetKnownTypes", typeof(KnownTypeProvider))]
        Employee GetEmployee(int businessEntityId);

        /// <summary>
        /// get list of entities 'EmployeeList' using a Where clause
		/// Database Table Employee
        /// </summary>
        /// <returns>Entity 'EmployeeList' with matched result</returns>
        [OperationContract]
		[WebInvoke(UriTemplate = "/GetEmployeeList", RequestFormat = WebMessageFormat.Json, ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
		[ServiceKnownType("GetKnownTypes", typeof(KnownTypeProvider))]
        EmployeeList GetEmployeeList(SqlWhereClause where);

        /// <summary>
        /// Save or create an entity 'Employee', accourding to entity content and state.
		/// Database Table Employee
        /// </summary>
        /// <returns>Entity 'Employee', the saved/created entity, proof <see href="ResultInfo.IsValid"/></returns>
        [OperationContract]
		[WebInvoke(UriTemplate = "/SaveEmployee", RequestFormat = WebMessageFormat.Json, ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
		[ServiceKnownType("GetKnownTypes", typeof(KnownTypeProvider))]
        Employee SaveEmployee(Employee item);

        /// <summary>
        /// Remove existing entity 'Employee'
		/// Database Table Employee
        /// </summary>
        /// <returns>true, when removed, false otherwise</returns>
        [OperationContract]
		[WebInvoke(UriTemplate = "/RemoveEmployee", RequestFormat = WebMessageFormat.Json, ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
		[ServiceKnownType("GetKnownTypes", typeof(KnownTypeProvider))]
        bool RemoveEmployee(Employee item);
    }

    public partial class AdventureWorksService
    {
        /// <summary>
        /// <see href="IAdventureWorksService.GetEmployee" />
		/// Database Table Employee
        /// </summary>
        virtual public Employee GetEmployee(int businessEntityId)
        {
		    Employee result = null;
			this.Execute((manager) =>
			{
				result = manager.GetEmployee(businessEntityId);
			});
			return result;
        }

        /// <summary>
        /// <see href="IAdventureWorksService.GetEmployeeList" />
		/// Database Table Employee
        /// </summary>
        virtual public EmployeeList GetEmployeeList(SqlWhereClause where)
        {
			EmployeeList result = null;
			this.Execute((manager) =>
			{
				result = manager.GetEmployeeList(where);
			});
			return result;
        }

        /// <summary>
        /// <see href="IAdventureWorksService.SaveEmployee" />
		/// Database Table Employee
        /// </summary>
        virtual public Employee SaveEmployee(Employee item)
        {
			Employee result = null;
			this.Execute((manager) =>
			{
				result = manager.SaveEmployee(item);
			});
			return result;
        }
		
        /// <summary>
        /// <see href="IAdventureWorksService.RemoveEmployee" />
		/// Database Table Employee
        /// </summary>
        virtual public bool RemoveEmployee(Employee item)
        {
			bool result = false;
			this.Execute((manager) =>
			{
				result = manager.RemoveEmployee(item);
			});
			return result;
        }
    }
}