﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using System.Web;
using System.ServiceModel.Web;

using Wellcode.Basics;
using Wellcode.Frame;

namespace AdventureWorks
{
    public partial interface IAdventureWorksService
    {
		/// <summary>
        /// getting existing entity 'Person' using it's primary key(s)
		/// Tabelle Person
        /// </summary>
        /// <returns>a new object of entity 'Person', <see href="ResultInfo.IsValid"/></returns>
        [OperationContract]
		[WebInvoke(UriTemplate = "/GetPerson", Method = "POST", RequestFormat = WebMessageFormat.Json, ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
		[ServiceKnownType("GetKnownTypes", typeof(KnownTypeProvider))]
        Person GetPerson(int businessEntityId);

        /// <summary>
        /// get list of entities 'PersonList' using a Where clause
		/// Database Table Person
        /// </summary>
        /// <returns>Entity 'PersonList' with matched result</returns>
        [OperationContract]
		[WebInvoke(UriTemplate = "/GetPersonList", RequestFormat = WebMessageFormat.Json, ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
		[ServiceKnownType("GetKnownTypes", typeof(KnownTypeProvider))]
        PersonList GetPersonList(SqlWhereClause where);

        /// <summary>
        /// Save or create an entity 'Person', accourding to entity content and state.
		/// Database Table Person
        /// </summary>
        /// <returns>Entity 'Person', the saved/created entity, proof <see href="ResultInfo.IsValid"/></returns>
        [OperationContract]
		[WebInvoke(UriTemplate = "/SavePerson", RequestFormat = WebMessageFormat.Json, ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
		[ServiceKnownType("GetKnownTypes", typeof(KnownTypeProvider))]
        Person SavePerson(Person item);

        /// <summary>
        /// Remove existing entity 'Person'
		/// Database Table Person
        /// </summary>
        /// <returns>true, when removed, false otherwise</returns>
        [OperationContract]
		[WebInvoke(UriTemplate = "/RemovePerson", RequestFormat = WebMessageFormat.Json, ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
		[ServiceKnownType("GetKnownTypes", typeof(KnownTypeProvider))]
        bool RemovePerson(Person item);
    }

    public partial class AdventureWorksService
    {
        /// <summary>
        /// <see href="IAdventureWorksService.GetPerson" />
		/// Database Table Person
        /// </summary>
        virtual public Person GetPerson(int businessEntityId)
        {
		    Person result = null;
			this.Execute((manager) =>
			{
				result = manager.GetPerson(businessEntityId);
			});
			return result;
        }

        /// <summary>
        /// <see href="IAdventureWorksService.GetPersonList" />
		/// Database Table Person
        /// </summary>
        virtual public PersonList GetPersonList(SqlWhereClause where)
        {
			PersonList result = null;
			this.Execute((manager) =>
			{
				result = manager.GetPersonList(where);
			});
			return result;
        }

        /// <summary>
        /// <see href="IAdventureWorksService.SavePerson" />
		/// Database Table Person
        /// </summary>
        virtual public Person SavePerson(Person item)
        {
			Person result = null;
			this.Execute((manager) =>
			{
				result = manager.SavePerson(item);
			});
			return result;
        }
		
        /// <summary>
        /// <see href="IAdventureWorksService.RemovePerson" />
		/// Database Table Person
        /// </summary>
        virtual public bool RemovePerson(Person item)
        {
			bool result = false;
			this.Execute((manager) =>
			{
				result = manager.RemovePerson(item);
			});
			return result;
        }
    }
}