﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using System.Web;
using System.ServiceModel.Web;

using Wellcode.Basics;
using Wellcode.Frame;

namespace AdventureWorks
{
    public partial interface IAdventureWorksService
    {
		/// <summary>
        /// getting existing entity 'Product' using it's primary key(s)
		/// Tabelle Product
        /// </summary>
        /// <returns>a new object of entity 'Product', <see href="ResultInfo.IsValid"/></returns>
        [OperationContract]
		[WebInvoke(UriTemplate = "/GetProduct", Method = "POST", RequestFormat = WebMessageFormat.Json, ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
		[ServiceKnownType("GetKnownTypes", typeof(KnownTypeProvider))]
        Product GetProduct(int productId);

        /// <summary>
        /// get list of entities 'ProductList' using a Where clause
		/// Database Table Product
        /// </summary>
        /// <returns>Entity 'ProductList' with matched result</returns>
        [OperationContract]
		[WebInvoke(UriTemplate = "/GetProductList", RequestFormat = WebMessageFormat.Json, ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
		[ServiceKnownType("GetKnownTypes", typeof(KnownTypeProvider))]
        ProductList GetProductList(SqlWhereClause where);

        /// <summary>
        /// Save or create an entity 'Product', accourding to entity content and state.
		/// Database Table Product
        /// </summary>
        /// <returns>Entity 'Product', the saved/created entity, proof <see href="ResultInfo.IsValid"/></returns>
        [OperationContract]
		[WebInvoke(UriTemplate = "/SaveProduct", RequestFormat = WebMessageFormat.Json, ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
		[ServiceKnownType("GetKnownTypes", typeof(KnownTypeProvider))]
        Product SaveProduct(Product item);

        /// <summary>
        /// Remove existing entity 'Product'
		/// Database Table Product
        /// </summary>
        /// <returns>true, when removed, false otherwise</returns>
        [OperationContract]
		[WebInvoke(UriTemplate = "/RemoveProduct", RequestFormat = WebMessageFormat.Json, ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
		[ServiceKnownType("GetKnownTypes", typeof(KnownTypeProvider))]
        bool RemoveProduct(Product item);
    }

    public partial class AdventureWorksService
    {
        /// <summary>
        /// <see href="IAdventureWorksService.GetProduct" />
		/// Database Table Product
        /// </summary>
        virtual public Product GetProduct(int productId)
        {
		    Product result = null;
			this.Execute((manager) =>
			{
				result = manager.GetProduct(productId);
			});
			return result;
        }

        /// <summary>
        /// <see href="IAdventureWorksService.GetProductList" />
		/// Database Table Product
        /// </summary>
        virtual public ProductList GetProductList(SqlWhereClause where)
        {
			ProductList result = null;
			this.Execute((manager) =>
			{
				result = manager.GetProductList(where);
			});
			return result;
        }

        /// <summary>
        /// <see href="IAdventureWorksService.SaveProduct" />
		/// Database Table Product
        /// </summary>
        virtual public Product SaveProduct(Product item)
        {
			Product result = null;
			this.Execute((manager) =>
			{
				result = manager.SaveProduct(item);
			});
			return result;
        }
		
        /// <summary>
        /// <see href="IAdventureWorksService.RemoveProduct" />
		/// Database Table Product
        /// </summary>
        virtual public bool RemoveProduct(Product item)
        {
			bool result = false;
			this.Execute((manager) =>
			{
				result = manager.RemoveProduct(item);
			});
			return result;
        }
    }
}