﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using System.Web;
using System.ServiceModel.Web;

using Wellcode.Basics;
using Wellcode.Frame;

namespace AdventureWorks
{
    public partial interface IAdventureWorksService
    {
		/// <summary>
        /// getting existing entity 'Store' using it's primary key(s)
		/// Tabelle Store
        /// </summary>
        /// <returns>a new object of entity 'Store', <see href="ResultInfo.IsValid"/></returns>
        [OperationContract]
		[WebInvoke(UriTemplate = "/GetStore", Method = "POST", RequestFormat = WebMessageFormat.Json, ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
		[ServiceKnownType("GetKnownTypes", typeof(KnownTypeProvider))]
        Store GetStore(int businessEntityId);

        /// <summary>
        /// get list of entities 'StoreList' using a Where clause
		/// Database Table Store
        /// </summary>
        /// <returns>Entity 'StoreList' with matched result</returns>
        [OperationContract]
		[WebInvoke(UriTemplate = "/GetStoreList", RequestFormat = WebMessageFormat.Json, ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
		[ServiceKnownType("GetKnownTypes", typeof(KnownTypeProvider))]
        StoreList GetStoreList(SqlWhereClause where);

        /// <summary>
        /// Save or create an entity 'Store', accourding to entity content and state.
		/// Database Table Store
        /// </summary>
        /// <returns>Entity 'Store', the saved/created entity, proof <see href="ResultInfo.IsValid"/></returns>
        [OperationContract]
		[WebInvoke(UriTemplate = "/SaveStore", RequestFormat = WebMessageFormat.Json, ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
		[ServiceKnownType("GetKnownTypes", typeof(KnownTypeProvider))]
        Store SaveStore(Store item);

        /// <summary>
        /// Remove existing entity 'Store'
		/// Database Table Store
        /// </summary>
        /// <returns>true, when removed, false otherwise</returns>
        [OperationContract]
		[WebInvoke(UriTemplate = "/RemoveStore", RequestFormat = WebMessageFormat.Json, ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
		[ServiceKnownType("GetKnownTypes", typeof(KnownTypeProvider))]
        bool RemoveStore(Store item);
    }

    public partial class AdventureWorksService
    {
        /// <summary>
        /// <see href="IAdventureWorksService.GetStore" />
		/// Database Table Store
        /// </summary>
        virtual public Store GetStore(int businessEntityId)
        {
		    Store result = null;
			this.Execute((manager) =>
			{
				result = manager.GetStore(businessEntityId);
			});
			return result;
        }

        /// <summary>
        /// <see href="IAdventureWorksService.GetStoreList" />
		/// Database Table Store
        /// </summary>
        virtual public StoreList GetStoreList(SqlWhereClause where)
        {
			StoreList result = null;
			this.Execute((manager) =>
			{
				result = manager.GetStoreList(where);
			});
			return result;
        }

        /// <summary>
        /// <see href="IAdventureWorksService.SaveStore" />
		/// Database Table Store
        /// </summary>
        virtual public Store SaveStore(Store item)
        {
			Store result = null;
			this.Execute((manager) =>
			{
				result = manager.SaveStore(item);
			});
			return result;
        }
		
        /// <summary>
        /// <see href="IAdventureWorksService.RemoveStore" />
		/// Database Table Store
        /// </summary>
        virtual public bool RemoveStore(Store item)
        {
			bool result = false;
			this.Execute((manager) =>
			{
				result = manager.RemoveStore(item);
			});
			return result;
        }
    }
}