﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using System.Web;
using System.ServiceModel.Web;

using Wellcode.Basics;
using Wellcode.Frame;

namespace AdventureWorks
{
    public partial interface IAdventureWorksService
    {
		/// <summary>
        /// getting existing entity 'WorkOrder' using it's primary key(s)
		/// Tabelle WorkOrder
        /// </summary>
        /// <returns>a new object of entity 'WorkOrder', <see href="ResultInfo.IsValid"/></returns>
        [OperationContract]
		[WebInvoke(UriTemplate = "/GetWorkOrder", Method = "POST", RequestFormat = WebMessageFormat.Json, ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
		[ServiceKnownType("GetKnownTypes", typeof(KnownTypeProvider))]
        WorkOrder GetWorkOrder(int workOrderId);

        /// <summary>
        /// get list of entities 'WorkOrderList' using a Where clause
		/// Database Table WorkOrder
        /// </summary>
        /// <returns>Entity 'WorkOrderList' with matched result</returns>
        [OperationContract]
		[WebInvoke(UriTemplate = "/GetWorkOrderList", RequestFormat = WebMessageFormat.Json, ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
		[ServiceKnownType("GetKnownTypes", typeof(KnownTypeProvider))]
        WorkOrderList GetWorkOrderList(SqlWhereClause where);

        /// <summary>
        /// Save or create an entity 'WorkOrder', accourding to entity content and state.
		/// Database Table WorkOrder
        /// </summary>
        /// <returns>Entity 'WorkOrder', the saved/created entity, proof <see href="ResultInfo.IsValid"/></returns>
        [OperationContract]
		[WebInvoke(UriTemplate = "/SaveWorkOrder", RequestFormat = WebMessageFormat.Json, ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
		[ServiceKnownType("GetKnownTypes", typeof(KnownTypeProvider))]
        WorkOrder SaveWorkOrder(WorkOrder item);

        /// <summary>
        /// Remove existing entity 'WorkOrder'
		/// Database Table WorkOrder
        /// </summary>
        /// <returns>true, when removed, false otherwise</returns>
        [OperationContract]
		[WebInvoke(UriTemplate = "/RemoveWorkOrder", RequestFormat = WebMessageFormat.Json, ResponseFormat = WebMessageFormat.Json, BodyStyle = WebMessageBodyStyle.Wrapped)]
		[ServiceKnownType("GetKnownTypes", typeof(KnownTypeProvider))]
        bool RemoveWorkOrder(WorkOrder item);
    }

    public partial class AdventureWorksService
    {
        /// <summary>
        /// <see href="IAdventureWorksService.GetWorkOrder" />
		/// Database Table WorkOrder
        /// </summary>
        virtual public WorkOrder GetWorkOrder(int workOrderId)
        {
		    WorkOrder result = null;
			this.Execute((manager) =>
			{
				result = manager.GetWorkOrder(workOrderId);
			});
			return result;
        }

        /// <summary>
        /// <see href="IAdventureWorksService.GetWorkOrderList" />
		/// Database Table WorkOrder
        /// </summary>
        virtual public WorkOrderList GetWorkOrderList(SqlWhereClause where)
        {
			WorkOrderList result = null;
			this.Execute((manager) =>
			{
				result = manager.GetWorkOrderList(where);
			});
			return result;
        }

        /// <summary>
        /// <see href="IAdventureWorksService.SaveWorkOrder" />
		/// Database Table WorkOrder
        /// </summary>
        virtual public WorkOrder SaveWorkOrder(WorkOrder item)
        {
			WorkOrder result = null;
			this.Execute((manager) =>
			{
				result = manager.SaveWorkOrder(item);
			});
			return result;
        }
		
        /// <summary>
        /// <see href="IAdventureWorksService.RemoveWorkOrder" />
		/// Database Table WorkOrder
        /// </summary>
        virtual public bool RemoveWorkOrder(WorkOrder item)
        {
			bool result = false;
			this.Execute((manager) =>
			{
				result = manager.RemoveWorkOrder(item);
			});
			return result;
        }
    }
}